load("multimodal center.Rdata")
center  # the mean vectors for the components
library("tmvtnorm")

####################
## To estimate the true mean and covariance, you can use the following code, which run a long MC chain ##
# library("tmvtnorm")
# x=matrix(0,10000001,2)
# for(i in 1:10000000){
#   id=sample(1:10,1,replace=T)
#   x[i,]=rtmvnorm(1,mean=center[id,],sigma=diag(0.01,2),lower=rep(0,2),upper=rep(10,2))
# }
# my.mean=apply(x,2,mean)
# my.var=apply(x,2,var)
# my.cov=cov(x[,1],x[,2])
# my.mean
# my.var
# my.cov
######################

## target distribution ##
f=function(x){
  result=sum(apply(center,1,function(t){dtnorm(x[1],t[1],0.1,lower=0,upper=10)*dtnorm(x[2],t[2],0.1,lower=0,upper=10)}))
  return (result)
}

e_dist=function(x){
  sqrt(sum(x^2))
}

find=function(x){
  flag=apply(center,1,function(z){any(apply((x-z)^2,1,e_dist)<=0.4)})
  sum(flag)
}
# require package "msm" 
library("msm")

## ordinary MCMC ##
N=10000
x=matrix(0,N+1,2);
x[1,]=c(2.5,3.5)# initial point (2.5,3.5)
u=runif(N,0,1)
for (i in 1:N){
  temp.x=NULL
  temp.x[1]=rtnorm(1,x[i,1],0.5,lower=0,upper=10)
  temp.x[2]=rtnorm(1,x[i,2],0.5,lower=0,upper=10)
  if (u[i]<f(temp.x)*dtnorm(x[i,1],temp.x[1],0.5,lower=0,upper=10)*dtnorm(x[i,2],temp.x[2],0.5,lower=0,upper=10)/f(x[i,])/dtnorm(temp.x[1],x[i,1],0.5,lower=0,upper=10)/dtnorm(temp.x[2],x[i,2],0.5,lower=0,upper=10)) {x[i+1,]=temp.x} else {x[i+1,]=x[i,]}
}
apply(x,2,mean)
apply(x,2,var)
cov(x[,1],x[,2])
plot(x[seq(1,10001,20),],pch=20,xlab="X",ylab="Y",main="",xlim=c(0,10),ylim=c(0,10),col="grey")
points(center,pch=20)

# trace plot
plot(c(0,0),xlab="X",ylab="Y",xlim=c(0,10),ylim=c(0,10),type="n")
lines(x[1:10000,],lty=1)


## SMSS algorithms ##
multi=function(x,s)
{
  length(which(colSums(t(s)-x)==0))
}

## SMS-MWG ## 
N=10000;K=5;
x=matrix(0,N+1,2);
u=runif(N,0,1);
x[1,]=c(2.5,3.5);S=matrix(rep(x[1,],K),K,2,byrow=T);
n1=n2=0
for(i in 1:N){
  id=sample(1:K,1);
  tmp=S;
  tmp[id,1]=rtnorm(1,S[id,1],0.5,lower=0,upper=10)
  tmp[id,2]=rtnorm(1,S[id,2],0.5,lower=0,upper=10)
  if(u[i]<=multi(x[i,],tmp)*multi(tmp[id,],tmp)*dtnorm(S[id,1],tmp[id,1],0.5,lower=0,upper=10)*dtnorm(S[id,2],tmp[id,2],0.5,lower=0,upper=10)/multi(x[i,],S)/multi(S[id,],S)/dtnorm(tmp[id,1],S[id,1],0.5,lower=0,upper=10)/dtnorm(tmp[id,2],S[id,2],0.5,lower=0,upper=10)) {S=tmp;n1=n1+1;} 
  prop=apply(S,1,f);
  x.id=sample(1:K,1,prob=prop);
  x[i+1,]=S[x.id,];
  if(!all(x[i+1,]==x[i,])) n2=n2+1
}
apply(x,2,mean)
apply(x,2,var)
cov(x[,1],x[,2])
plot(x[seq(1,10001,20),],pch=20,xlab="X",ylab="Y",main="",xlim=c(0,10),ylim=c(0,10),col="grey")
points(center,pch=20)


## SMS-CBMC ## 
N=10000;K=5;
x=matrix(0,N+1,2);
u=runif(N,0,1);
x[1,]=c(2.5,3.5);S=matrix(rep(x[1,],K),K,2,byrow=T);
count=0
for(i in 1:N){
  id=sample(1:K,1);
  tmp=S;
  tmp[id,1]=rtnorm(1,S[id,1],0.5,lower=0,upper=10)
  tmp[id,2]=rtnorm(1,S[id,2],0.5,lower=0,upper=10)
  if(u[i]<=sum(apply(tmp,1,f))*multi(tmp[id,],tmp)*dtnorm(S[id,1],tmp[id,1],0.5,lower=0,upper=10)*dtnorm(S[id,2],tmp[id,2],0.5,lower=0,upper=10)/sum(apply(S,1,f))/multi(S[id,],S)/dtnorm(tmp[id,1],S[id,1],0.5,lower=0,upper=10)/dtnorm(tmp[id,2],S[id,2],0.5,lower=0,upper=10)) {
    S=tmp; 
    prop=apply(S,1,f);
    x.id=sample(1:K,1,prob=prop);
    x[i+1,]=S[x.id,];
    count=count+1;
  } else x[i+1,]=x[i,];
}
apply(x,2,mean)
apply(x,2,var)
cov(x[,1],x[,2])
plot(x[seq(1,10001,20),],pch=20,xlab="X",ylab="Y",main="",xlim=c(0,10),ylim=c(0,10),col="grey")
points(center,pch=20)


## R-SMS-CBMC ## 
N=10000;K=5;
x=matrix(0,N+1,2);
#S=matrix(0,2,K);
u=runif(N,0,1);
x[1,]=c(2.5,3.5);S=matrix(rep(x[1,],K),K,2,byrow=T);
for(i in 1:N){
  id=sample(1:K,1);
  tmp=S;
  tmp[id,1]=rtnorm(1,S[id,1],0.5,lower=0,upper=10)
  tmp[id,2]=rtnorm(1,S[id,2],0.5,lower=0,upper=10)
  if(u[i]<=sum(apply(tmp,1,f))*multi(tmp[id,],tmp)*dtnorm(S[id,1],tmp[id,1],0.5,lower=0,upper=10)*dtnorm(S[id,2],tmp[id,2],0.5,lower=0,upper=10)/sum(apply(S,1,f))/multi(S[id,],S)/dtnorm(tmp[id,1],S[id,1],0.5,lower=0,upper=10)/dtnorm(tmp[id,2],S[id,2],0.5,lower=0,upper=10)) S=tmp; 
  prop=apply(S,1,f);
  x.id=sample(1:K,1,prob=prop);
  x[i+1,]=S[x.id,];
}
apply(x,2,mean)
apply(x,2,var)
cov(x[,1],x[,2])
plot(x[seq(1,10001,20),],pch=20,xlab="X",ylab="Y",main="",xlim=c(0,10),ylim=c(0,10),col="grey")
points(center,pch=20)

# trace plot
plot(c(0,0),xlab="X",ylab="Y",xlim=c(0,10),ylim=c(0,10),type="n")
lines(x[1:10000,],lty=1)


## parallel tempering ##
sigma=0.5
N=10^4
swap=0.5
K=5
inver_t=seq(1,0.02,length.out=K)
x=array(0,c(K,N+1,2))
x[,1,]=c(2.5,3.5)
iter=1
while(iter<=N){
  u=runif(1,0,1)
  if (u<=swap){
      for (i in 1:K){
          tmp=NULL
          tmp[1]=rtnorm(1,x[i,iter,1],sigma,lower=0,upper=10)
          tmp[2]=rtnorm(1,x[i,iter,2],sigma,lower=0,upper=10)
          if(runif(1,0,1)<=f(tmp)^inver_t[i]*dtnorm(x[i,iter,1],tmp[1],sigma,lower=0,upper=10)*dtnorm(x[i,iter,2],tmp[2],sigma,lower=0,upper=10)/(f(x[i,iter,])^inver_t[i])/dtnorm(tmp[1],x[i,iter,1],sigma,lower=0,upper=10)/dtnorm(tmp[2],x[i,iter,2],sigma,lower=0,upper=10)){
            x[i,iter+1,]=tmp
          } else {
            x[i,iter+1,]=x[i,iter,]
          }
        }
        iter=iter+1
  } else {
        x[,iter+1,]=x[,iter,]
        j=sample(K-1,1)
        if(runif(1,0,1)<=f(x[j+1,iter,])^inver_t[j]*f(x[j,iter,])^inver_t[j+1]/f(x[j,iter,])^inver_t[j]/f(x[j+1,iter,])^inver_t[j+1]){
          temp=x[j,iter+1,]
          x[j,iter+1,]=x[j+1,iter+1,]
          x[j+1,iter+1,]=temp
        }
        iter=iter+1
  }
}
num_mode=find(x[1,,])
num_mode

## EMC ##
crossover=function(x,y,k){
  base=10^(k-1)
  res2=floor(x*base)/base+y-floor(y*base)/base
  res1=floor(y*base)/base+x-floor(x*base)/base
  return(list(res1,res2))
}

sigma=0.5
n=10^4
mutation=0.3
N=5
inver_t=seq(1,0.02,length.out=N)
x=array(0,c(N,n+1,2))
x[,1,]=c(2.5,3.5);
iter=1
while(iter<=n){
  u=runif(1,0,1)
  if (u<=mutation){
      for (z in 1:N){
          tmp=NULL
          tmp[1]=rtnorm(1,x[z,iter,1],sigma,lower=0,upper=10)
          tmp[2]=rtnorm(1,x[z,iter,2],sigma,lower=0,upper=10)
          if(runif(1,0,1)<=f(tmp)^inver_t[z]*dtnorm(x[z,iter,1],tmp[1],sigma,lower=0,upper=10)*dtnorm(x[z,iter,2],tmp[2],sigma,lower=0,upper=10)/(f(x[z,iter,])^inver_t[z])/dtnorm(tmp[1],x[z,iter,1],sigma,lower=0,upper=10)/dtnorm(tmp[2],x[z,iter,2],sigma,lower=0,upper=10)){
            x[z,iter+1,]=tmp
          } else {
            x[z,iter+1,]=x[z,iter,]
          }
      }
      iter=iter+1
    } else {
        #crossover
        x[,iter+1,]=x[,iter,]
        fitness=apply(x[,iter,],1,f)
        pool=1:N
        i=sample(pool,1,prob=fitness)
        j=sample(pool[pool!=i],1)
        res_co1=crossover(x[i,iter,1],x[j,iter,1],sample(2,1))
        res_co2=crossover(x[i,iter,2],x[j,iter,2],sample(2,1))
        yi=c(res_co1[[1]],res_co2[[1]])
        yj=c(res_co1[[2]],res_co2[[2]])
        zx=sum(fitness)
        zy=zx-f(x[i,iter,])-f(x[j,iter,])+f(yi)+f(yj)
        if(zy!=0 & runif(1,0,1)<=(f(yi)/f(x[i,iter,]))^inver_t[i]*(f(yj)/f(x[j,iter,]))^inver_t[j]*(f(yi)+f(yj))/zy*zx/(f(x[i,iter,])+f(x[j,iter,]))){
          x[i,iter+1,]=yi
          x[j,iter+1,]=yj
        }
        iter=iter+1
      }
      #exchange
      s=sample(N,1)
      if(s==1) t=s+1 else{
        t=ifelse(s==N,s-1,sample(c(s-1,s+1),1))
      }
      if(runif(1,0,1)<=(f(x[s,iter,])/f(x[t,iter,]))^(inver_t[t]-inver_t[s])){
        temp=x[t,iter,]
        x[t,iter,]=x[s,iter,]
        x[s,iter,]=temp
      }
}
num_mode=find(x[1,,])
num_mode
